@@ -1,3 +1,4 @@
+import pytest
 from bluetooth_sensor_state_data import BluetoothServiceInfo, SensorUpdate
 from sensor_state_data import (
     BinarySensorDescription,
@@ -18,7 +19,19 @@ def test_can_create():
     ThermoBeaconBluetoothDeviceData()
 
 
-MFR_20 = BluetoothServiceInfo(
+MFR_16_LEN_18 = BluetoothServiceInfo(
+    name="ThermoBeacon",
+    address="aa:bb:cc:dd:ee:ff",
+    rssi=-60,
+    service_data={},
+    manufacturer_data={
+        16: b"\x00\x00\x9c\r\x00\x00\xfeDE\x0c\xc5\x01\x87\x02\x17\x00\x00\x00"
+    },
+    service_uuids=["0000fff0-0000-1000-8000-00805f9b34fb"],
+    source="local",
+)
+
+MFR_16_LEN_20 = BluetoothServiceInfo(
     name="ThermoBeacon",
     address="aa:bb:cc:dd:ee:ff",
     rssi=-60,
@@ -42,6 +55,18 @@ MFR_48 = BluetoothServiceInfo(
     source="local",
 )
 
+MFR_20 = BluetoothServiceInfo(
+    name="ThermoBeacon",
+    address="aa:bb:cc:dd:ee:ff",
+    rssi=-60,
+    service_data={},
+    manufacturer_data={
+        20: b"\x00\x00\x05\x03\x00\x00\xf5\xeb3\rV\x01<\x03L\x03\x00\x00"
+    },
+    service_uuids=["0000fff0-0000-1000-8000-00805f9b34fb"],
+    source="local",
+)
+
 MFR_22 = BluetoothServiceInfo(
     name="ThermoBeacon",
     address="aa:bb:cc:dd:ee:ff",
@@ -89,23 +114,40 @@ BAD_DATA = BluetoothServiceInfo(
     service_uuids=["0000fff0-0000-1000-8000-00805f9b34fb"],
     source="local",
 )
+BAD_DATA_2 = BluetoothServiceInfo(
+    name="ThermoBeacon",
+    address="aa:bb:cc:dd:ee:ff",
+    rssi=-60,
+    service_data={},
+    manufacturer_data={
+        16: b"\x00\x00\x16\x00\x00\xac\xfa/\x0b&\x01G\x03\xa7\xe7\x12\x00\xc0"
+    },
+    service_uuids=["0000fff0-0000-1000-8000-00805f9b34fb"],
+    source="local",
+)
 
 
 def test_with_22_byte_update():
     parser = ThermoBeaconBluetoothDeviceData()
-    parser.supported(MFR_22) is True
+    assert parser.supported(MFR_22) is True
     assert parser.title == "Smart hygrometer EEFF"
 
 
 def test_supported_set_the_title():
     parser = ThermoBeaconBluetoothDeviceData()
-    parser.supported(MFR_20) is True
+    assert parser.supported(MFR_16_LEN_20) is True
+    assert parser.title == "Lanyard/mini hygrometer EEFF"
+
+
+def test_supported_set_the_title_18_bytes():
+    parser = ThermoBeaconBluetoothDeviceData()
+    assert parser.supported(MFR_16_LEN_18) is True
     assert parser.title == "Lanyard/mini hygrometer EEFF"
 
 
 def test_20_byte_update():
     parser = ThermoBeaconBluetoothDeviceData()
-    update = parser.update(MFR_20)
+    update = parser.update(MFR_16_LEN_20)
     assert update == SensorUpdate(
         title="Lanyard/mini hygrometer EEFF",
         devices={
@@ -187,6 +229,90 @@ def test_20_byte_update():
     )
 
 
+def test_18_byte_update():
+    parser = ThermoBeaconBluetoothDeviceData()
+    update = parser.update(MFR_16_LEN_18)
+    assert update == SensorUpdate(
+        title="Lanyard/mini hygrometer EEFF",
+        devices={
+            None: SensorDeviceInfo(
+                name="Lanyard/mini hygrometer EEFF",
+                model=16,
+                manufacturer="ThermoBeacon",
+                sw_version=None,
+                hw_version=None,
+            )
+        },
+        entity_descriptions={
+            DeviceKey(key="humidity", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="humidity", device_id=None),
+                device_class=SensorDeviceClass.HUMIDITY,
+                native_unit_of_measurement=Units.PERCENTAGE,
+            ),
+            DeviceKey(key="voltage", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="voltage", device_id=None),
+                device_class=SensorDeviceClass.VOLTAGE,
+                native_unit_of_measurement=Units.ELECTRIC_POTENTIAL_VOLT,
+            ),
+            DeviceKey(key="temperature", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="temperature", device_id=None),
+                device_class=SensorDeviceClass.TEMPERATURE,
+                native_unit_of_measurement=Units.TEMP_CELSIUS,
+            ),
+            DeviceKey(key="battery", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="battery", device_id=None),
+                device_class=SensorDeviceClass.BATTERY,
+                native_unit_of_measurement=Units.PERCENTAGE,
+            ),
+            DeviceKey(key="signal_strength", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="signal_strength", device_id=None),
+                device_class=SensorDeviceClass.SIGNAL_STRENGTH,
+                native_unit_of_measurement=Units.SIGNAL_STRENGTH_DECIBELS_MILLIWATT,
+            ),
+        },
+        entity_values={
+            DeviceKey(key="humidity", device_id=None): SensorValue(
+                device_key=DeviceKey(key="humidity", device_id=None),
+                name="Humidity",
+                native_value=40.44,
+            ),
+            DeviceKey(key="voltage", device_id=None): SensorValue(
+                device_key=DeviceKey(key="voltage", device_id=None),
+                name="Voltage",
+                native_value=3.14,
+            ),
+            DeviceKey(key="temperature", device_id=None): SensorValue(
+                device_key=DeviceKey(key="temperature", device_id=None),
+                name="Temperature",
+                native_value=28.31,
+            ),
+            DeviceKey(key="battery", device_id=None): SensorValue(
+                device_key=DeviceKey(key="battery", device_id=None),
+                name="Battery",
+                native_value=100,
+            ),
+            DeviceKey(key="signal_strength", device_id=None): SensorValue(
+                device_key=DeviceKey(key="signal_strength", device_id=None),
+                name="Signal " "Strength",
+                native_value=-60,
+            ),
+        },
+        binary_entity_descriptions={
+            DeviceKey(key="occupancy", device_id=None): BinarySensorDescription(
+                device_key=DeviceKey(key="occupancy", device_id=None),
+                device_class=BinarySensorDeviceClass.OCCUPANCY,
+            )
+        },
+        binary_entity_values={
+            DeviceKey(key="occupancy", device_id=None): BinarySensorValue(
+                device_key=DeviceKey(key="occupancy", device_id=None),
+                name="Occupancy",
+                native_value=False,
+            )
+        },
+    )
+
+
 def test_mfr_27():
     parser = ThermoBeaconBluetoothDeviceData()
     update = parser.update(MFR_27)
@@ -357,9 +483,101 @@ def test_mfr_48():
     )
 
 
-def test_bad_data_ignored():
+def test_mfr_20():
     parser = ThermoBeaconBluetoothDeviceData()
-    update = parser.update(BAD_DATA)
+    update = parser.update(MFR_20)
+    assert update == SensorUpdate(
+        title="Smart hygrometer EEFF",
+        devices={
+            None: SensorDeviceInfo(
+                name="Smart hygrometer EEFF",
+                model=20,
+                manufacturer="ThermoBeacon",
+                sw_version=None,
+                hw_version=None,
+            )
+        },
+        entity_descriptions={
+            DeviceKey(key="battery", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="battery", device_id=None),
+                device_class=SensorDeviceClass.BATTERY,
+                native_unit_of_measurement=Units.PERCENTAGE,
+            ),
+            DeviceKey(key="signal_strength", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="signal_strength", device_id=None),
+                device_class=SensorDeviceClass.SIGNAL_STRENGTH,
+                native_unit_of_measurement=Units.SIGNAL_STRENGTH_DECIBELS_MILLIWATT,
+            ),
+            DeviceKey(key="temperature", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="temperature", device_id=None),
+                device_class=SensorDeviceClass.TEMPERATURE,
+                native_unit_of_measurement=Units.TEMP_CELSIUS,
+            ),
+            DeviceKey(key="voltage", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="voltage", device_id=None),
+                device_class=SensorDeviceClass.VOLTAGE,
+                native_unit_of_measurement=Units.ELECTRIC_POTENTIAL_VOLT,
+            ),
+            DeviceKey(key="humidity", device_id=None): SensorDescription(
+                device_key=DeviceKey(key="humidity", device_id=None),
+                device_class=SensorDeviceClass.HUMIDITY,
+                native_unit_of_measurement=Units.PERCENTAGE,
+            ),
+        },
+        entity_values={
+            DeviceKey(key="battery", device_id=None): SensorValue(
+                device_key=DeviceKey(key="battery", device_id=None),
+                name="Battery",
+                native_value=100,
+            ),
+            DeviceKey(key="signal_strength", device_id=None): SensorValue(
+                device_key=DeviceKey(key="signal_strength", device_id=None),
+                name="Signal " "Strength",
+                native_value=-60,
+            ),
+            DeviceKey(key="temperature", device_id=None): SensorValue(
+                device_key=DeviceKey(key="temperature", device_id=None),
+                name="Temperature",
+                native_value=21.38,
+            ),
+            DeviceKey(key="voltage", device_id=None): SensorValue(
+                device_key=DeviceKey(key="voltage", device_id=None),
+                name="Voltage",
+                native_value=3.38,
+            ),
+            DeviceKey(key="humidity", device_id=None): SensorValue(
+                device_key=DeviceKey(key="humidity", device_id=None),
+                name="Humidity",
+                native_value=51.75,
+            ),
+        },
+        binary_entity_descriptions={
+            DeviceKey(key="occupancy", device_id=None): BinarySensorDescription(
+                device_key=DeviceKey(key="occupancy", device_id=None),
+                device_class=BinarySensorDeviceClass.OCCUPANCY,
+            )
+        },
+        binary_entity_values={
+            DeviceKey(key="occupancy", device_id=None): BinarySensorValue(
+                device_key=DeviceKey(key="occupancy", device_id=None),
+                name="Occupancy",
+                native_value=False,
+            )
+        },
+        events={},
+    )
+
+
+@pytest.mark.parametrize(
+    "data",
+    [
+        BAD_DATA,
+        BAD_DATA_2,
+    ],
+)
+def test_bad_data_ignored(data: bytes) -> None:
+    parser = ThermoBeaconBluetoothDeviceData()
+    update = parser.update(data)
     assert update == SensorUpdate(
         title="Lanyard/mini hygrometer EEFF",
         devices={
